@@ -1,11 +1,9 @@
-# $Id: TCP.pm 2569 2009-07-14 21:40:50Z bingosnet $
-
 package POE::Component::Server::TCP;
 
 use strict;
 
 use vars qw($VERSION);
-$VERSION = do {my($r)=(q$Revision: 2569 $=~/(\d+)/);sprintf"1.%04d",$r};
+$VERSION = '1.269_001'; # NOTE - Should be #.### (three decimal places)
 
 use Carp qw(carp croak);
 use Socket qw(INADDR_ANY inet_ntoa inet_aton AF_INET AF_UNIX PF_UNIX);
@@ -48,7 +46,8 @@ sub new {
 
   foreach (
     qw(
-      Acceptor Error ClientInput ClientConnected ClientDisconnected
+      Acceptor Error ClientInput
+      ClientPreConnect ClientConnected ClientDisconnected
       ClientError ClientFlushed
       ClientLow ClientHigh
     )
@@ -89,6 +88,7 @@ sub new {
     }
   }
 
+  my $client_pre_connect  = delete $param{ClientPreConnect};
   my $client_connected    = delete $param{ClientConnected};
   my $client_disconnected = delete $param{ClientDisconnected};
   my $client_error        = delete $param{ClientError};
@@ -115,7 +115,7 @@ sub new {
     }
   }
 
-  my $client_args         = delete($param{ClientArgs}) || delete($param{Args});
+  my $client_args = delete($param{ClientArgs}) || delete($param{Args});
 
   # Defaults.
 
@@ -128,7 +128,6 @@ sub new {
   $address = INADDR_ANY unless defined $address;
 
   $error_callback = \&_default_server_error unless defined $error_callback;
-  $server_started = sub {} unless ref($server_started) eq 'CODE';
 
   $session_type = 'POE::Session' unless defined $session_type;
   if (defined($session_params) && ref($session_params)) {
@@ -141,10 +140,7 @@ sub new {
 
   if (defined $client_input) {
     $client_error  = \&_default_client_error unless defined $client_error;
-    $client_connected    = sub {} unless defined $client_connected;
-    $client_disconnected = sub {} unless defined $client_disconnected;
-    $client_flushed      = sub {} unless defined $client_flushed;
-    $client_args = [] unless defined $client_args;
+    $client_args         = []     unless defined $client_args;
 
     # Extra states.
 
@@ -243,7 +239,6 @@ sub new {
       $accept_callback = sub {
         my ($socket, $remote_addr, $remote_port) = @_[ARG0, ARG1, ARG2];
 
-
         $session_type->create(
           @$session_params,
           inline_states => {
@@ -275,8 +270,18 @@ sub new {
 
               $heap->{remote_port} = $remote_port;
 
+              my $socket = $_[ARG0];
+              if ($client_pre_connect) {
+                $socket = $client_pre_connect->(@_);
+                unless (fileno($socket)) {
+                  # TODO - Error callback?  Disconnected callback?
+                  # TODO - Should we do this before starting the child?
+                  return;
+                }
+              }
+
               $heap->{client} = POE::Wheel::ReadWrite->new(
-                Handle       => splice(@_, ARG0, 1),
+                Handle       => $socket,
                 Driver       => POE::Driver::SysRW->new(),
                 _get_filters(
                   $client_filter,
@@ -299,7 +304,11 @@ sub new {
                 ),
               );
 
-              $client_connected->(@_);
+              # Expand the Args constructor array, and place a copy
+              # into @_[ARG0..].  There are only 2 parameters.
+              splice(@_, ARG0, 2, @{$_[ARG1]});
+
+              $client_connected and $client_connected->(@_);
             },
             tcp_server_got_high => $high_event,
             tcp_server_got_low => $low_event,
@@ -327,10 +336,10 @@ sub new {
             tcp_server_got_flush => sub {
               my $heap = $_[HEAP];
               DEBUG and warn "$$: $alias child Flush";
-              $client_flushed->(@_);
+              $client_flushed and $client_flushed->(@_);
               if ($heap->{shutdown}) {
                 DEBUG and warn "$$: $alias child Flush, callback";
-                $client_disconnected->(@_);
+                $client_disconnected and $client_disconnected->(@_);
                 delete $heap->{client};
               }
             },
@@ -344,7 +353,7 @@ sub new {
                   not $heap->{client}->get_driver_out_octets()
                 ) {
                   DEBUG and warn "$$: $alias child Shutdown, callback";
-                  $client_disconnected->(@_);
+                  $client_disconnected and $client_disconnected->(@_);
                   delete $heap->{client};
                 }
               }
@@ -377,6 +386,8 @@ sub new {
           package_states => $package_states,
           object_states  => $object_states,
 
+          # XXX - If you change the number of args here, also change
+          # the splice elsewhere.
           args => [ $socket, $client_args ],
         );
       };
@@ -435,7 +446,7 @@ sub new {
           SuccessEvent => 'tcp_server_got_connection',
           FailureEvent => 'tcp_server_got_error',
         );
-        $server_started->(@_);
+        $server_started and $server_started->(@_);
       },
       # Catch an error.
       tcp_server_got_error => $error_callback,
@@ -573,7 +584,7 @@ sub _test_filter {
     if (!$eval and $@) {
         carp(
           "Failed to load [$filter]\n" .
-          "Reason $@\nUsing defualt POE::Filter::Line "
+          "Reason $@\nUsing default POE::Filter::Line "
         );
         return 0;
     }
@@ -670,12 +681,27 @@ POE::Wheel::SocketFactory's C<SuccessEvent>.
 Otherwise, the default C<Acceptor> callback will start a new session
 to handle each connection.  These child sessions do not have their own
 aliases, but their C<ClientConnected> and C<ClientDisconnected>
-callbacks may register and unregister the sessions with a shared
-namespace of their own.
+callbacks may be used to register and unregister the sessions with a
+shared namespace, such as a hash keyed on session IDs, or an object
+that manages such a hash.
+
+  my %client_namespace;
+
+  sub handle_client_connected {
+    my $client_session_id = $_[SESSION]->ID;
+    $client_namespace{$client_session_id} = \%anything;
+  }
+
+  sub handle_client_disconnected {
+    my $client_session_id = $_[SESSION]->ID;
+    $client_namespace{$client_session_id} = \%anything;
+  }
 
 The component's C<Started> callback is invoked at the end of the
-master session's startup routine.  This callback's parameters are
-the usual ones for C<_start>.
+master session's startup routine.  The @_[ARG0..$#_] parameters are
+set to a copy of the values in the server's C<ListenerArgs>
+constructor parameter.  The other parameters are standard for
+POE::Session's _start handlers.
 
 The component's C<Error> callback is invoked when the server has a
 problem listening for connections.  C<Error> may also be called if the
@@ -704,12 +730,41 @@ The component's C<ClientInput> callback defines how child sessions
 will handle input from their clients.  Its parameters are that of
 POE::Wheel::ReadWrite's C<InputEvent>.
 
-C<ClientConnected> is called at the end of the child session's
-C<_start> routine.  In addition to the usual C<_start> parameters, it
-includes the socket in $_[ARG0] and the contents of the component's
-C<Args> constructor parameter in $_[ARG1].
+As mentioned C<ClientConnected> is called at the end of the child
+session's C<_start> routine.  The C<ClientConneted> callback receives
+the same parameters as the client session's _start does.  The arrayref
+passed to the constructor's C<Args> parameter is flattened and
+included in C<ClientConnected>'s parameters as @_[ARG0..$#_].
 
-TODO - Should C<Args> be flattened into C<ARG1..$%_>?
+  sub handle_client_connected {
+    my @constructor_args = @_[ARG0..$#_];
+    ...
+  }
+
+C<ClientPreConnect> is called before C<ClientConnected>, and it has
+different parameters: $_[ARG0] contains a copy of the client socket
+before it's given to POE::Wheel::ReadWrite for management.  Most HEAP
+members are set, except of course $_[HEAP]{client}, because the
+POE::Wheel::ReadWrite has not yet been created yet.
+C<ClientPreConnect> may enable SSL on the socket, using
+POE::Component::SSLify.  C<ClientPreConnect> must return a valid
+socket to complete the connection; the client will be disconnected if
+anything else is returned.
+
+  sub handle_client_pre_connect {
+
+    # Make sure the remote address and port are valid.
+    return undef unless validate(
+      $_[HEAP]{remote_ip}, $_[HEAP]{remote_port}
+    );
+
+    # SSLify the socket, which is in $_[ARG0].
+    my $socket = eval { Server_SSLify($_[ARG0]) };
+    return undef if $@;
+
+    # Return the SSL-ified socket.
+    return $socket;
+  }
 
 C<ClientDisconnected> is called when the client has disconnected,
 either because the remote socket endpoint has closed or the local
@@ -982,9 +1037,8 @@ interact with established connections.
 =head4 ClientArgs
 
 C<ClientArgs> is optional.  When specified, it holds an ARRAYREF that
-will be passed to the C<ClientConnected> callback in $_[ARG1].
-(ClientConnected's $_[ARG0] contains the newly accepted client
-socket.)
+will be expanded one level and passed to the C<ClientConnected>
+callback in @_[ARG0..$#_].
 
 =head4 ClientConnected
 
@@ -993,9 +1047,9 @@ C<ClientConnected> is a callback that notifies the application when a
 client's session is started and ready for operation.  Banners are
 often sent to the remote client from this callback.
 
-C<ClientConnected> callbacks receive the usual POE parameters plus:
-The newly accepted client socket in $_[ARG0] and the ARRAYREF
-specified in C<ClientArgs> in $_[ARG1].
+The @_[ARG0..$#_] parameters to C<ClientConnected> are a copy of the
+values in the C<ClientArgs> constructor parameter's array referece.
+The other @_ members are standard for a POE::Session _start handler.
 
 C<ClientConnected> is called once per session startup.  It will never
 be called twice for the same connection.
@@ -1005,8 +1059,6 @@ be called twice for the same connection.
     # Other client initialization here.
   },
 
-TODO - Show Args and client socket introspection.
-
 =head4 ClientDisconnected
 
 C<ClientDisconnected> is a callback that will be invoked when the
@@ -1331,8 +1383,6 @@ Some use cases require different session classes for the listener and
 the connection handlers.  This isn't currently supported.  Please send
 patches. :)
 
-TODO - Rename C<Args> into C<ClientArgs>.
-
 TODO - Document that Reuse is set implicitly.
 
 =head1 AUTHORS & COPYRIGHTS
