@@ -1,5 +1,3 @@
-# $Id: PerlSignals.pm 2585 2009-07-21 02:22:15Z rcaputo $
-
 # Plain Perl signal handling is something shared by several event
 # loops.  The invariant code has moved out here so that each loop may
 # use it without reinventing it.  This will save maintenance and
@@ -10,7 +8,7 @@ package POE::Loop::PerlSignals;
 use strict;
 
 use vars qw($VERSION);
-$VERSION = do {my($r)=(q$Revision: 2585 $=~/(\d+)/);sprintf"1.%04d",$r};
+$VERSION = '1.269_001'; # NOTE - Should be #.### (three decimal places)
 
 # Everything plugs into POE::Kernel.
 package POE::Kernel;
@@ -83,8 +81,7 @@ sub _loop_signal_handler_chld_bottom {
     POE::Kernel::_warn "<sg> Enqueuing CHLD-like SIG$_[0] event";
   }
 
-  $poe_kernel->_data_sig_enqueue_poll_event();
-  $SIG{$_[0]} = \&_loop_signal_handler_chld;
+  $poe_kernel->_data_sig_enqueue_poll_event($_[0]);
 }
 
 #------------------------------------------------------------------------------
@@ -98,18 +95,13 @@ sub loop_watch_signal {
   # Child process has stopped.
   if ($signal eq 'CHLD' or $signal eq 'CLD') {
     if ( USE_SIGCHLD ) {
-      # install, but also trigger once
-      # there may be a race condition between forking, and $kernel->sig_chld in
-      # which the signal is already delivered
-      # and the interval polling mechanism will still generate a SIGCHLD
-      # signal, this preserves that behavior
-      $SIG{$signal} = \&_loop_signal_handler_chld;
-      $self->_data_sig_enqueue_poll_event();
+      # Poll once for signals.  Will set the signal handler when done.
+      $self->_data_sig_enqueue_poll_event($signal);
     } else {
       # We should never twiddle $SIG{CH?LD} under POE, unless we want to
       # override system() and friends. --hachi
       # $SIG{$signal} = "DEFAULT";
-      $self->_data_sig_begin_polling();
+      $self->_data_sig_begin_polling($signal);
     }
     return;
   }
@@ -129,7 +121,7 @@ sub loop_ignore_signal {
 
   delete $signal_watched{$signal};
 
-    if ($signal eq 'CHLD' or $signal eq 'CLD') {
+  if ($signal eq 'CHLD' or $signal eq 'CLD') {
     if ( USE_SIGCHLD ) {
       if( $self->_data_sig_child_procs) {
         # We need SIGCHLD to stay around after shutdown, so that
