@@ -1,13 +1,12 @@
-# $Id: Array.pm 2197 2007-07-22 02:21:32Z rcaputo $
 # Copyrights and documentation are at the end.
 
 package POE::Queue::Array;
 
 use strict;
 
-use vars qw(@ISA $VERSION);
+use vars qw($VERSION @ISA);
+$VERSION = '1.269_001'; # NOTE - Should be #.### (three decimal places)
 @ISA = qw(POE::Queue);
-$VERSION = do {my($r)=(q$Revision: 2197 $=~/(\d+)/);sprintf"1.%04d",$r};
 
 use Errno qw(ESRCH EPERM);
 use Carp qw(confess);
@@ -279,7 +278,7 @@ sub adjust_priority {
   }
 
   # Nothing to do if the delta is zero.
-  # -><- Actually we may need to ensure that the item is moved to the
+  # TODO Actually we may need to ensure that the item is moved to the
   # end of its current priority bucket, since it should have "moved".
   return $self->[$item_index]->[ITEM_PRIORITY] unless $delta;
 
@@ -305,7 +304,7 @@ sub set_priority {
   }
 
   # Nothing to do if the old and new priorities match.
-  # -><- Actually we may need to ensure that the item is moved to the
+  # TODO Actually we may need to ensure that the item is moved to the
   # end of its current priority bucket, since it should have "moved".
   return $new_priority if $new_priority == $old_priority;
 
@@ -443,11 +442,13 @@ See L<POE::Queue>.
 =head1 DESCRIPTION
 
 This class is an implementation of the abstract POE::Queue interface.
-It implement the priority queue using Perl arrays, splice, and a
-copious application of cleverness.
+As such, its documentation may be found in L<POE::Queue>.
 
-Please see the L<POE::Queue> documentation, which explains this one's
-functions, features, and behavior.
+POE::Queue::Array implements a priority queue using Perl arrays,
+splice, and copious application of cleverness.
+
+Despite its name, POE::Queue::Array may be used as a stand-alone
+priority queue without the rest of POE.
 
 =head1 SEE ALSO
 
@@ -463,3 +464,6 @@ Please see L<POE> for more information about authors, contributors,
 and POE's licensing.
 
 =cut
+
+# rocco // vim: ts=2 sw=2 expandtab
+# TODO - Edit.
