@@ -1,12 +1,10 @@
-# $Id: Aliases.pm 1903 2006-03-20 04:44:08Z rcaputo $
-
 # Manage the POE::Kernel data structures necessary to keep track of
 # session aliases.
 
-package POE::Resources::Aliases;
+package POE::Resource::Aliases;
 
 use vars qw($VERSION);
-$VERSION = do {my($r)=(q$Revision: 1903 $=~/(\d+)/);sprintf"1.%04d",$r};
+$VERSION = '1.269_001'; # NOTE - Should be #.### (three decimal places)
 
 # These methods are folded into POE::Kernel;
 package POE::Kernel;
@@ -28,10 +26,9 @@ my %kr_ses_to_alias;
 #    ...,
 #  );
 
-sub _data_ses_initialize {
+sub _data_alias_initialize {
   $poe_kernel->[KR_ALIASES] = \%kr_aliases;
 }
-use POE::API::ResLoader \&_data_ses_initialize;
 
 ### End-run leak checking.  Returns true if finalization was ok, or
 ### false if it failed.
@@ -52,14 +49,14 @@ sub _data_alias_finalize {
 
 # Add an alias to a session.
 #
-# -><- This has a potential problem: setting the same alias twice on a
+# TODO This has a potential problem: setting the same alias twice on a
 # session will increase the session's reference count twice.  Removing
 # the alias will only decrement it once.  That potentially causes
 # reference counts that never go away.  The public interface for this
 # function, alias_set(), does not allow this to occur.  We should add
 # a test to make sure it never does.
 #
-# -><- It is possible to add aliases to sessions that do not exist.
+# TODO It is possible to add aliases to sessions that do not exist.
 # The public alias_set() function prevents this from happening.
 
 sub _data_alias_add {
@@ -71,7 +68,7 @@ sub _data_alias_add {
 
 # Remove an alias from a session.
 #
-# -><- Happily allows the removal of aliases from sessions that don't
+# TODO Happily allows the removal of aliases from sessions that don't
 # exist.  This will cause problems with reference counting.
 
 sub _data_alias_remove {
@@ -138,28 +135,38 @@ __END__
 
 =head1 NAME
 
-POE::Resources::Aliases - manage session aliases for POE::Kernel
+POE::Resource::Aliases - internal session alias manager for POE::Kernel
 
 =head1 SYNOPSIS
 
-Used internally by POE::Kernel.  Better documentation will be
-forthcoming.
+There is no public API.
 
 =head1 DESCRIPTION
 
-This module manages session aliases for POE::Kernel.  It is used
-internally by POE::Kernel and has no public interface.
+POE::Resource::Aliases is a mix-in class for POE::Kernel.  It provides
+the features to manage session aliases.  It is used internally by
+POE::Kernel, so it has no public interface.
 
 =head1 SEE ALSO
 
-See L<POE::Kernel> for documentation on session aliases.
+See L<POE::Kernel/Session Identifiers (IDs and Aliases)> for the
+public alias API.
+
+See L<POE::Kernel/Resources> for for public information about POE
+resources.
+
+See L<POE::Resource> for general discussion about resources and the
+classes that manage them.
 
 =head1 BUGS
 
-Probably.
+None known.
 
 =head1 AUTHORS & COPYRIGHTS
 
 Please see L<POE> for more information about authors and contributors.
 
 =cut
+
+# rocco // vim: ts=2 sw=2 expandtab
+# TODO - Edit.
