@@ -1,11 +1,9 @@
-# $Id: Events.pm 2447 2009-02-17 05:04:43Z rcaputo $
-
 # Data and accessors to manage POE's events.
 
 package POE::Resource::Events;
 
 use vars qw($VERSION);
-$VERSION = do {my($r)=(q$Revision: 2447 $=~/(\d+)/);sprintf"1.%04d",$r};
+$VERSION = '1.269_001'; # NOTE - Should be #.### (three decimal places)
 
 # These methods are folded into POE::Kernel;
 package POE::Kernel;
@@ -52,8 +50,9 @@ sub _data_ev_finalize {
 
 sub _data_ev_enqueue {
   my (
-    $self, $session, $source_session, $event, $type, $etc, $file, $line,
-    $fromstate, $time
+    $self,
+    $session, $source_session, $event, $type, $etc,
+    $file, $line, $fromstate, $time
   ) = @_;
 
   if (ASSERT_DATA) {
@@ -89,11 +88,11 @@ sub _data_ev_enqueue {
   # This is the counterpart to _data_ev_refcount_dec().  It's only
   # used in one place, so it's not in its own function.
 
-  $self->_data_ses_refcount_inc($session);
   $event_count{$session}++;
+  $self->_data_ses_refcount_inc($session);
 
-  $self->_data_ses_refcount_inc($source_session);
   $post_count{$source_session}++;
+  $self->_data_ses_refcount_inc($source_session);
 
   return $new_id;
 }
@@ -204,11 +203,11 @@ sub _data_ev_refcount_dec {
     _trap $source_session unless exists $post_count{$source_session};
   }
 
-  $self->_data_ses_refcount_dec($dest_session);
   $event_count{$dest_session}--;
+  $self->_data_ses_refcount_dec($dest_session);
 
-  $self->_data_ses_refcount_dec($source_session);
   $post_count{$source_session}--;
+  $self->_data_ses_refcount_dec($source_session);
 }
 
 ### Fetch the number of pending events sent to a session.
@@ -261,27 +260,24 @@ sub _data_ev_dispatch_due {
       $self->_data_stat_add('blocked_seconds', $now - $due_time);
     }
 
+    # TODO - Can these two lines be reversed?
+    # TODO - May avoid entering/removing GC mark entries.
     $self->_data_ev_refcount_dec($event->[EV_SOURCE], $event->[EV_SESSION]);
     $self->_dispatch_event(@$event, $due_time, $id);
 
-    # An exception occurred.
-    if ($POE::Kernel::kr_exception) {
-
-      # Save the exception lexically, then clear it so it doesn't
-      # linger if run() is called again.
-      my $exception = $POE::Kernel::kr_exception;
-      $POE::Kernel::kr_exception = undef;
-
-      # Stop the kernel.  Cleans out all sessions.
-      POE::Kernel->stop();
-
-      # Throw the exception from way out here.
-      die $exception;
-    }
+    # Stop the system if an unhandled exception occurred.
+    # This wipes out all sessions and associated resources.
+    POE::Kernel->stop() if $POE::Kernel::kr_exception;
   }
 
+  # Sweep for dead sessions.  The sweep may alter the next queue time.
+
+  $self->_data_ses_gc_sweep();
+  $next_time = $kr_queue->get_next_priority();
+
   # Tell the event loop to wait for the next event, if there is one.
   # Otherwise we're going to wait indefinitely for some other event.
+
   if (defined $next_time) {
     $self->loop_reset_time_watcher($next_time);
   }
