@@ -1,12 +1,10 @@
-# $Id: Signals.pm 2595 2009-07-22 19:03:28Z rcaputo $
-
 # The data necessary to manage signals, and the accessors to get at
 # that data in a sane fashion.
 
 package POE::Resource::Signals;
 
 use vars qw($VERSION);
-$VERSION = do {my($r)=(q$Revision: 2595 $=~/(\d+)/);sprintf"1.%04d",$r};
+$VERSION = '1.269_001'; # NOTE - Should be #.### (three decimal places)
 
 # These methods are folded into POE::Kernel;
 package POE::Kernel;
@@ -64,7 +62,8 @@ use vars (
 #my $kr_signal_type;                 # The type of signal being dispatched.
 
 # A flag to tell whether we're currently polling for signals.
-# Under USE_SIGCHLD, determines whether a SIGCHLD polling event has already been queued
+# Under USE_SIGCHLD, determines whether a SIGCHLD polling event has
+# already been queued.
 my $polling_for_signals = 0;
 
 # A flag determining whether there are child processes.
@@ -156,9 +155,11 @@ sub _data_sig_has_forked {
 }
 
 sub _data_sig_reset_procs {
+  my $self = shift;
   # Initialize this to a true value so our waitpid() loop can run at
   # least once.  Starts false when running in an Apache handler so our
   # SIGCHLD hijinks don't interfere with the web server.
+  $self->_data_sig_cease_polling();
   $kr_child_procs = exists($INC{'Apache.pm'}) ? 0 : ( USE_SIGCHLD ? 0 : 1 );
 }
 
@@ -170,10 +171,13 @@ sub _data_sig_get_safe_signals {
 }
 
 ### End-run leak checking.
+our $finalizing;
 
 sub _data_sig_finalize {
   my( $self ) = @_;
   my $finalized_ok = 1;
+  # tell _data_sig_pipe_send to ignore CHLD that waitpid might provoke
+  local $finalizing = 1;
 
   $self->_data_sig_pipe_finalize;
 
@@ -214,6 +218,7 @@ sub _data_sig_finalize {
     local $?;
 
     my $leaked_children = 0;
+
     until ((my $pid = waitpid( -1, 0 )) == -1) {
       _warn( "!!! Child process PID:$pid reaped: $!\n" ) if $pid;
       $finalized_ok = 0;
@@ -451,19 +456,6 @@ sub _data_sig_free_terminated_sessions {
       $self->_data_ses_stop($dead_session);
     }
   }
-  else {
-    # TODO Implicit signal reaping.  This is deprecated behavior and
-    # will eventually be removed.  See the commented out tests in
-    # t/res/signals.t.
-    #
-    # Don't reap the parent if it's the kernel.  It still needs to be
-    # a part of the system for finalization in certain cases.
-    foreach my $touched_session (@kr_signaled_sessions) {
-      next unless $self->_data_ses_exists($touched_session);
-      next if $touched_session == $self;
-      $self->_data_ses_collect_garbage($touched_session);
-    }
-  }
 
   # Erase @kr_signaled_sessions, or they will leak until the next
   # signal.
@@ -480,12 +472,12 @@ sub _data_sig_touched_session {
 
 # only used under !USE_SIGCHLD
 sub _data_sig_begin_polling {
-  my $self = shift;
+  my ($self, $signal) = @_;
 
   return if $polling_for_signals;
   $polling_for_signals = 1;
 
-  $self->_data_sig_enqueue_poll_event();
+  $self->_data_sig_enqueue_poll_event($signal);
   $self->_idle_queue_grow();
 }
 
@@ -495,14 +487,14 @@ sub _data_sig_cease_polling {
 }
 
 sub _data_sig_enqueue_poll_event {
-  my $self = shift;
+  my ($self, $signal) = @_;
 
   if ( USE_SIGCHLD ) {
     return if $polling_for_signals;
     $polling_for_signals = 1;
 
     $self->_data_ev_enqueue(
-      $self, $self, EN_SCPOLL, ET_SCPOLL, [ ],
+      $self, $self, EN_SCPOLL, ET_SCPOLL, [ $signal ],
       __FILE__, __LINE__, undef, time(),
     );
   } else {
@@ -510,21 +502,24 @@ sub _data_sig_enqueue_poll_event {
     return unless $polling_for_signals;
 
     $self->_data_ev_enqueue(
-      $self, $self, EN_SCPOLL, ET_SCPOLL, [ ],
+      $self, $self, EN_SCPOLL, ET_SCPOLL, [ $signal ],
       __FILE__, __LINE__, undef, time() + POE::Kernel::CHILD_POLLING_INTERVAL(),
     );
   }
 }
 
 sub _data_sig_handle_poll_event {
-  my $self = shift;
+  my ($self, $signal) = @_;
 
   if ( USE_SIGCHLD ) {
     $polling_for_signals = undef;
   }
 
   if (TRACE_SIGNALS) {
-    _warn("<sg> POE::Kernel is polling for signals at " . time() . (USE_SIGCHLD ? " due to SIGCHLD" : ""));
+    _warn(
+      "<sg> POE::Kernel is polling for signals at " . time() .
+      (USE_SIGCHLD ? " due to SIGCHLD" : "")
+    );
   }
 
   # Reap children for as long as waitpid(2) says something
@@ -622,18 +617,28 @@ sub _data_sig_handle_poll_event {
 
   $kr_child_procs = !$pid;
 
-
-  unless ( USE_SIGCHLD ) {
+  if (USE_SIGCHLD) {
+    if (TRACE_SIGNALS) {
+      _warn("<sg> POE::Kernel has reset the SIG$signal handler");
+    }
+    # Per https://rt.cpan.org/Ticket/Display.html?id=45109 setting the
+    # signal handler must be done after reaping the outstanding child
+    # processes, at least on SysV systems like HP-UX.
+    $SIG{$signal} = \&_loop_signal_handler_chld;
+  }
+  else {
     # The poll loop is over.  Resume slowly polling for signals.
 
-    if (TRACE_SIGNALS) {
-      _warn("<sg> POE::Kernel will poll again after a delay");
-    }
-
     if ($polling_for_signals) {
-      $self->_data_sig_enqueue_poll_event();
+      if (TRACE_SIGNALS) {
+        _warn("<sg> POE::Kernel will poll again after a delay");
+      }
+      $self->_data_sig_enqueue_poll_event($signal);
     }
     else {
+      if (TRACE_SIGNALS) {
+        _warn("<sg> POE::Kernel SIGCHLD poll loop paused");
+      }
       $self->_idle_queue_shrink();
     }
   }
@@ -737,8 +742,8 @@ sub _data_sig_mask_all {
   unless( $signal_mask_all ) {
     $self->_data_sig_mask_build;
   }
-  my $mask_temp;
-  sigprocmask( SIG_SETMASK, $signal_mask_all, $mask_temp ) 
+  my $mask_temp = POSIX::SigSet->new();
+  sigprocmask( SIG_SETMASK, $signal_mask_all, $mask_temp )
             or _trap "<sg> Unable to mask all signals: $!";
 }
 
@@ -749,8 +754,8 @@ sub _data_sig_unmask_all {
   unless( $signal_mask_none ) {
     $self->_data_sig_mask_build;
   }
-  my $mask_temp;
-  sigprocmask( SIG_SETMASK, $signal_mask_none, $mask_temp ) 
+  my $mask_temp = POSIX::SigSet->new();
+  sigprocmask( SIG_SETMASK, $signal_mask_none, $mask_temp )
         or _trap "<sg> Unable to unmask all signals: $!";
 }
 
@@ -765,11 +770,12 @@ sub _data_sig_pipe_finalize {
   if( $signal_pipe_write ) {
     close $signal_pipe_write; undef $signal_pipe_write;
   }
-  undef $signal_pipe_pid;
+  # Don't send anything more!
+  undef( $signal_pipe_pid );
 }
 
 ### Send a signal "message" to the main thread
-### Called from the bottom signal handlers
+### Called from the top signal handlers
 sub _data_sig_pipe_send {
   my $n = $SIG2NUM{ $_[1] };
   if( ASSERT_DATA ) {
@@ -778,9 +784,16 @@ sub _data_sig_pipe_send {
   if( TRACE_SIGNALS ) {
     _warn "<sg> Caught SIG$_[1] ($n)";
   }
-  if( $$ != $signal_pipe_pid ) {
-    _trap "<sg> Kernel now running in a different process.  You must call call \$poe_kernel->has_forked in the child process.";
+  
+  return if $finalizing;
+  
+  if( not defined $signal_pipe_pid ) {
+    _trap "<sg> $$ _data_sig_pipe_send called before signal pipe was initilised.";
   }
+  if( $$ != $signal_pipe_pid ) {
+    _trap "<sg> Kernel now running in a different process (is=$$ was=$signal_pipe_pid).  You must call call \$poe_kernel->has_forked in the child process.";
+  }
+
 
   my $count = _data_sig_pipe_syswrite( pack( "C", $n ) );
   if( ASSERT_DATA ) {
@@ -811,7 +824,7 @@ sub _data_sig_pipe_syswrite {
 }
 
 ### Read all signal numbers.
-### Call the related top handlers
+### Call the related bottom handler.  That is, inside the kernel loop.
 sub _data_sig_pipe_read {
   my( $self, $fileno, $mode ) = @_;
   if( ASSERT_DATA ) {
