@@ -1,14 +1,13 @@
-# $Id: ReadWrite.pm 2454 2009-02-18 10:02:54Z bingosnet $
-
 package POE::Wheel::ReadWrite;
 
 use strict;
 
 use vars qw($VERSION);
-$VERSION = do {my($r)=(q$Revision: 2454 $=~/(\d+)/);sprintf"1.%04d",$r};
+$VERSION = '1.269_001'; # NOTE - Should be #.### (three decimal places)
 
 use Carp qw( croak carp );
 use POE qw(Wheel Driver::SysRW Filter::Line);
+use base qw(POE::Wheel);
 
 # Offsets into $self.
 sub HANDLE_INPUT               () {  0 }
@@ -390,12 +389,12 @@ sub DESTROY {
   # Turn off the select.  This is a problem if a wheel is being
   # swapped, since it will turn off selects for the other wheel.
   if ($self->[HANDLE_INPUT]) {
-    $poe_kernel->select($self->[HANDLE_INPUT]);
+    $poe_kernel->select_read($self->[HANDLE_INPUT]);
     $self->[HANDLE_INPUT] = undef;
   }
 
   if ($self->[HANDLE_OUTPUT]) {
-    $poe_kernel->select($self->[HANDLE_OUTPUT]);
+    $poe_kernel->select_write($self->[HANDLE_OUTPUT]);
     $self->[HANDLE_OUTPUT] = undef;
   }
 
@@ -658,7 +657,7 @@ __END__
 
 =head1 NAME
 
-POE::Wheel::ReadWrite - non-blocking buffered I/O mix-in for POE::Sessoin
+POE::Wheel::ReadWrite - non-blocking buffered I/O mix-in for POE::Session
 
 =head1 SYNOPSIS
 
