@@ -1,11 +1,10 @@
-# $Id: Run.pm 2447 2009-02-17 05:04:43Z rcaputo $
-
 package POE::Wheel::Run;
 
 use strict;
 
-use vars qw($VERSION);
-$VERSION = do {my($r)=(q$Revision: 2447 $=~/(\d+)/);sprintf"1.%04d",$r};
+use vars qw($VERSION @ISA);
+$VERSION = '1.269_001'; # NOTE - Should be #.### (three decimal places)
+@ISA = 'POE::Wheel';
 
 use Carp qw(carp croak);
 use POSIX qw(
@@ -14,6 +13,7 @@ use POSIX qw(
 );
 
 use POE qw( Wheel Pipe::TwoWay Pipe::OneWay Driver::SysRW Filter::Line );
+use base qw(POE::Wheel);
 
 BEGIN {
   die "$^O does not support fork()\n" if $^O eq 'MacOS';
@@ -27,13 +27,23 @@ BEGIN {
   }
 
   if (POE::Kernel::RUNNING_IN_HELL) {
-    eval    { require Win32::Console; };
-    if ($@) { die "Win32::Console failed to load:\n$@" }
-    else    { Win32::Console->import(); };
+    eval    { require Win32::Console; Win32::Console->import() };
+    if ($@) { die "Win32::Console needed for POE::Wheel::Run on $^O:\n$@" }
 
-    eval    { require Win32API::File; };
-    if ($@) { die "Win32API::File but failed to load:\n$@" }
-    else    { Win32API::File->import( qw(FdGetOsFHandle) ); };
+    eval    {
+      require Win32API::File;
+      Win32API::File->import("FdGetOsFHandle");
+    };
+    if ($@) { die "Win32::File needed for POE::Wheel::Run on $^O:\n$@" }
+
+    eval    { require Win32::Process; Win32::Process->import() };
+    if ($@) { die "Win32::Process needed for POE::Wheel::Run on $^O:\n$@" }
+
+    eval    { require Win32::Job; Win32::Job->import() };
+    if ($@) { die "Win32::Job needed for POE::Wheel::Run on $^O:\n$@" }
+
+    eval    { require Win32; Win32->import() };
+    if ($@) { die "Win32.pm needed for POE::Wheel::Run on $^O:\n$@" }
   }
 
   # Determine the most file descriptors we can use.
@@ -76,6 +86,8 @@ sub DRIVER_STDERR () { 22 }
 sub EVENT_STDERR  () { 23 }
 sub STATE_STDERR  () { 24 }
 
+sub MSWIN32_GROUP_PID () { 25 }
+
 # Used to work around a bug in older perl versions.
 sub CRIMSON_SCOPE_HACK ($) { 0 }
 
@@ -157,12 +169,10 @@ sub new {
       defined($stderr_event)
     );
 
-  my $stdio_driver  = delete $params{StdioDriver}
-    || POE::Driver::SysRW->new();
+  my $stdio_driver  = delete $params{StdioDriver}  || POE::Driver::SysRW->new();
   my $stdin_driver  = delete $params{StdinDriver}  || $stdio_driver;
   my $stdout_driver = delete $params{StdoutDriver} || $stdio_driver;
-  my $stderr_driver = delete $params{StderrDriver}
-    || POE::Driver::SysRW->new();
+  my $stderr_driver = delete $params{StderrDriver} || POE::Driver::SysRW->new();
 
   my $stdio_filter  = delete $params{Filter};
   my $stdin_filter  = delete $params{StdinFilter};
@@ -262,6 +272,13 @@ sub new {
     croak "unknown conduit type $conduit";
   }
 
+  # Block signals until safe
+  my $must_unmask;
+  if( $poe_kernel->can( '_data_sig_mask_all' ) ) {
+    $poe_kernel->_data_sig_mask_all;
+    $must_unmask = 1;
+  }
+
   # Fork!  Woo-hoo!
   my $pid = fork;
 
@@ -274,6 +291,7 @@ sub new {
       carp "Cannot redirect into tied STDOUT.  Untying it";
       untie *STDOUT;
     }
+
     if (tied *STDERR) {
       carp "Cannot redirect into tied STDERR.  Untying it";
       untie *STDERR;
@@ -306,6 +324,7 @@ sub new {
       eval { $stdin_read->clone_winsize_from(\*STDIN) };
     }
     else {
+      # TODO - Can this be block eval?  Or a do{} block?
       eval 'setpgrp(0,0)' unless $no_setpgrp;
     }
 
@@ -314,6 +333,7 @@ sub new {
     # using them.
     my @safe_signals = $poe_kernel->_data_sig_get_safe_signals();
     @SIG{@safe_signals} = ("DEFAULT") x @safe_signals;
+    $poe_kernel->_data_sig_unmask_all if $must_unmask;
 
     # TODO How to pass events to the parent process?  Maybe over a
     # expedited (OOB) filehandle.
@@ -355,68 +375,36 @@ sub new {
     close $stdout_read;
     close $stderr_read if defined $stderr_read;
 
-    # Win32 needs the stdio handles closed before they're reopened
-    # because the standard handles aren't dup()'d.
-
-    # Redirect STDIN from the read end of the stdin pipe.
-    close STDIN if POE::Kernel::RUNNING_IN_HELL;
-    open( STDIN, "<&" . fileno($stdin_read) )
-      or die "can't redirect STDIN in child pid $$: $!";
-
-    # Redirect STDOUT to the write end of the stdout pipe.
-    # The STDOUT_FILENO check snuck in on a patch.  I'm not sure why
-    # we care what the file descriptor is.
-    close STDOUT if POE::Kernel::RUNNING_IN_HELL;
-    open( STDOUT, ">&" . fileno($stdout_write) )
-      or die "can't redirect stdout in child pid $$: $!";
-
-    # Redirect STDERR to the write end of the stderr pipe.  If the
-    # stderr pipe's undef, then we use STDOUT.
-    # The STDERR_FILENO check snuck in on a patch.  I'm not sure why
-    # we care what the file descriptor is.
-    close STDERR if POE::Kernel::RUNNING_IN_HELL;
-    open( STDERR, ">&" . fileno($stderr_write) )
-      or die "can't redirect stderr in child: $!";
+    if (POE::Kernel::RUNNING_IN_HELL) {
+      __PACKAGE__->_redirect_child_stdio_in_hell(
+        $stdin_read, $stdout_write, $stderr_write
+      );
+    }
+    else {
+      __PACKAGE__->_redirect_child_stdio_sanely(
+        $stdin_read, $stdout_write, $stderr_write
+      );
+    }
 
     # Make STDOUT and/or STDERR auto-flush.
     select STDERR;  $| = 1;
     select STDOUT;  $| = 1;
 
-    # Tell the parent that the stdio has been set up.
-    close $sem_pipe_read;
-    print $sem_pipe_write "go\n";
-    close $sem_pipe_write;
+    # The child doesn't need to read from the semaphore pipe.
+    $sem_pipe_read = undef;
 
-    if (POE::Kernel::RUNNING_IN_HELL)  {
-      # The Win32 pseudo fork sets up the std handles in the child
-      # based on the true win32 handles For the exec these get
-      # remembered, so manipulation of STDIN/OUT/ERR is not enough.
-      # Only necessary for the exec, as Perl CODE subroutine goes
-      # through 0/1/2 which are correct.  But of course that coderef
-      # might invoke exec, so better do it regardless.
-      # HACK: Using Win32::Console as nothing else exposes SetStdHandle
-      Win32::Console::_SetStdHandle(
-        STD_INPUT_HANDLE(),
-        FdGetOsFHandle(fileno($stdin_read))
-      );
-      Win32::Console::_SetStdHandle(
-        STD_OUTPUT_HANDLE(),
-        FdGetOsFHandle(fileno($stdout_write))
-      );
-      Win32::Console::_SetStdHandle(
-        STD_ERROR_HANDLE(),
-        FdGetOsFHandle(fileno($stderr_write))
-      );
-    }
-
-    # Exec the program depending on its form.
+    # Run Perl code.  This is farily consistent across most systems.
     if (ref($program) eq 'CODE') {
 
+      # Tell the parent that the stdio has been set up.
+      print $sem_pipe_write "go\n";
+      close $sem_pipe_write;
+
       # Close any close-on-exec file descriptors.  Except STDIN,
       # STDOUT, and STDERR, of course.
       if ($close_on_call) {
         for (0..MAX_OPEN_FDS-1) {
-          next if fileno(STDIN) == $_;
+          next if fileno(STDIN)  == $_;
           next if fileno(STDOUT) == $_;
           next if fileno(STDERR) == $_;
           POSIX::close($_);
@@ -425,38 +413,50 @@ sub new {
 
       $program->(@$prog_args);
 
-      # In case flushing them wasn't good enough.
+      # Try to force stdio flushing.
+      close STDIN  if defined fileno(STDIN); # Voodoo?
       close STDOUT if defined fileno(STDOUT);
       close STDERR if defined fileno(STDERR);
 
-      # Try to exit without triggering END or object destructors.
-      # Give up with a plain exit if we must.
-      # But we can't _exit on Win32 because it KILLS ALL THREADS,
-      # including the parent "process".
-      unless (POE::Kernel::RUNNING_IN_HELL) {
-        eval { POSIX::_exit(0);  };
-        eval { kill KILL => $$;  };
-        eval { exec("$^X -e 0"); };
-      };
-      exit(0);
+      exit __PACKAGE__->_exit_child_any_way_we_can();
     }
-    else {
-      if (ref($program) eq 'ARRAY') {
-        exec(@$program, @$prog_args)
-          or die "can't exec (@$program) in child pid $$: $!";
-      }
-      else {
-        exec(join(" ", $program, @$prog_args))
-          or die "can't exec ($program) in child pid $$: $!";
-      }
+
+    # Execute an external program.  This gets weird.
+
+    # Windows! What I do for you!
+    __PACKAGE__->_exec_in_hell(
+      $close_on_call, $sem_pipe_write, $program, $prog_args
+    ) if POE::Kernel::RUNNING_IN_HELL;
+
+    # Everybody else seems sane.
+    # Tell the parent that the stdio has been set up.
+    print $sem_pipe_write "go\n";
+    close $sem_pipe_write;
+
+    # exec(ARRAY)
+    if (ref($program) eq 'ARRAY') {
+      exec(@$program, @$prog_args)
+        or die "can't exec (@$program) in child pid $$: $!";
     }
-    die "insanity check passed";
+
+    # exec(SCALAR)
+    exec(join(" ", $program, @$prog_args))
+      or die "can't exec ($program) in child pid $$: $!";
   }
 
   # Parent here.  Close what the parent won't need.
-  close $stdin_read   if defined $stdin_read;
-  close $stdout_write if defined $stdout_write;
-  close $stderr_write if defined $stderr_write;
+  defined($stdin_read)   and close $stdin_read;
+  defined($stdout_write) and close $stdout_write;
+  defined($stderr_write) and close $stderr_write;
+
+  # Also close any slave ptys
+  $stdout_read->close_slave() if (
+    defined $stdout_read and ref($stdout_read) eq 'IO::Pty'
+  );
+
+  $stderr_read->close_slave() if (
+    defined $stderr_read and ref($stderr_read) eq 'IO::Pty'
+  );
 
   my $active_count = 0;
   $active_count++ if $stdout_event and $stdout_read;
@@ -491,15 +491,21 @@ sub new {
     $stderr_driver, # DRIVER_STDERR
     $stderr_event,  # EVENT_STDERR
     undef,          # STATE_STDERR
+    undef,          # MSWIN32_GROUP_PID
   ], $type;
 
+  # PG- I suspect <> might need PIPE
+  $poe_kernel->_data_sig_unmask_all if $must_unmask;
+
   # Wait here while the child sets itself up.
+  $sem_pipe_write = undef;
   {
-    local $/ = "\n";
-    <$sem_pipe_read>;
+    local $/ = "\n";  # TODO - Needed?
+    my $chldout = <$sem_pipe_read>;
+    chomp $chldout;
+    $self->[MSWIN32_GROUP_PID] = $chldout if $chldout ne 'go';
   }
   close $sem_pipe_read;
-  close $sem_pipe_write;
 
   $self->_define_stdin_flusher();
   $self->_define_stdout_reader() if defined $stdout_read;
@@ -827,7 +833,7 @@ sub DESTROY {
 
   # Turn off the STDIN thing.
   if ($self->[HANDLE_STDIN]) {
-    $poe_kernel->select($self->[HANDLE_STDIN]);
+    $poe_kernel->select_write($self->[HANDLE_STDIN]);
     $self->[HANDLE_STDIN] = undef;
   }
   if ($self->[STATE_STDIN]) {
@@ -836,7 +842,7 @@ sub DESTROY {
   }
 
   if ($self->[HANDLE_STDOUT]) {
-    $poe_kernel->select($self->[HANDLE_STDOUT]);
+    $poe_kernel->select_read($self->[HANDLE_STDOUT]);
     $self->[HANDLE_STDOUT] = undef;
   }
   if ($self->[STATE_STDOUT]) {
@@ -845,7 +851,7 @@ sub DESTROY {
   }
 
   if ($self->[HANDLE_STDERR]) {
-    $poe_kernel->select($self->[HANDLE_STDERR]);
+    $poe_kernel->select_read($self->[HANDLE_STDERR]);
     $self->[HANDLE_STDERR] = undef;
   }
   if ($self->[STATE_STDERR]) {
@@ -1076,7 +1082,165 @@ sub PID {
 sub kill {
   my ($self, $signal) = @_;
   $signal = 'TERM' unless defined $signal;
-  eval { kill $signal, $self->[CHILD_PID] };
+  if ( $self->[MSWIN32_GROUP_PID] ) {
+    Win32::Process::KillProcess( $self->[MSWIN32_GROUP_PID], 293 );
+  }
+  else {
+    eval { kill $signal, $self->[CHILD_PID] };
+  }
+}
+
+### Internal helpers.
+
+sub _redirect_child_stdio_in_hell {
+  my ($class, $stdin_read, $stdout_write, $stderr_write) = @_;
+
+  # Win32 needs the stdio handles closed before they're reopened
+  # because the standard handles aren't dup()'d.
+
+  close STDIN;
+  close STDOUT;
+  close STDERR;
+
+  $class->_redirect_child_stdio_sanely(
+    $stdin_read, $stdout_write, $stderr_write
+  );
+
+  # The Win32 pseudo fork sets up the std handles in the child
+  # based on the true win32 handles.  The reopening of stdio
+  # handles isn't enough.  We must also set the underlying
+  # Win32 notion of these handles for completeness.
+  #
+  # Only necessary for the exec, as Perl CODE subroutine goes
+  # through 0/1/2 which are correct.  But of course that coderef
+  # might invoke exec, so better do it regardless.
+  #
+  # HACK: Using Win32::Console as nothing else exposes
+  # SetStdHandle
+  #
+  # TODO - https://rt.cpan.org/Ticket/Display.html?id=50068 claims
+  # that these _SetStdHandle() calls may leak memory.  Do we have
+  # alternatives?
+
+  Win32::Console::_SetStdHandle(
+    STD_INPUT_HANDLE(),
+    FdGetOsFHandle(fileno($stdin_read))
+  );
+
+  Win32::Console::_SetStdHandle(
+    STD_OUTPUT_HANDLE(),
+    FdGetOsFHandle(fileno($stdout_write))
+  );
+
+  Win32::Console::_SetStdHandle(
+    STD_ERROR_HANDLE(),
+    FdGetOsFHandle(fileno($stderr_write))
+  );
+}
+
+sub _redirect_child_stdio_sanely {
+  my ($class, $stdin_read, $stdout_write, $stderr_write) = @_;
+
+  # Redirect STDIN from the read end of the stdin pipe.
+  open( STDIN, "<&" . fileno($stdin_read) )
+    or die "can't redirect STDIN in child pid $$: $!";
+
+  # Redirect STDOUT to the write end of the stdout pipe.
+  open( STDOUT, ">&" . fileno($stdout_write) )
+    or die "can't redirect stdout in child pid $$: $!";
+
+  # Redirect STDERR to the write end of the stderr pipe.
+  open( STDERR, ">&" . fileno($stderr_write) )
+    or die "can't redirect stderr in child: $!";
+}
+
+sub _exit_child_any_way_we_can {
+  my $class = shift;
+
+  # On Windows, subprocesses run in separate threads.  All the "fancy"
+  # methods act on entire processes, so they also exit the parent.
+
+  unless (POE::Kernel::RUNNING_IN_HELL) {
+    # Try to avoid triggering END blocks and object destructors.
+    eval { POSIX::_exit(0);  };
+    eval { CORE::kill KILL => $$;  };
+    eval { exec("$^X -e 0"); };
+  }
+
+  # Do what we must.
+  exit(0);
+}
+
+# RUNNING_IN_HELL use Win32::Process to create a pucker new shiny
+# process. It'll inherit our processes handles which is neat.
+
+sub _exec_in_hell {
+  my (
+    $class, $close_on_call, $sem_pipe_write,
+    $program, $prog_args
+  ) = @_;
+
+  my $exitcode = 0;
+
+  # Close any close-on-exec file descriptors.
+  # Except STDIN, STDOUT, and STDERR, of course.
+
+  if ($close_on_call) {
+    for (0..MAX_OPEN_FDS-1) {
+      next if fileno(STDIN) == $_;
+      next if fileno(STDOUT) == $_;
+      next if fileno(STDERR) == $_;
+      POSIX::close($_);
+    }
+  }
+
+  my ($appname, $cmdline);
+
+  if (ref $program eq 'ARRAY') {
+    $appname = $program->[0];
+    $cmdline = join(
+      ' ',
+      map { /\s/ && ! /"/ ? qq{"$_"} : $_ }
+      (@$program, @$prog_args)
+    );
+  }
+  else {
+    $appname = undef;
+    $cmdline = join(
+      ' ', $program,
+      map { /\s/ && ! /"/ ? qq{"$_"} : $_ }
+      @$prog_args
+    );
+  }
+
+  my $w32job;
+
+  unless ( $w32job = Win32::Job->new() ) {
+    print $sem_pipe_write "go\n\n";
+    close $sem_pipe_write;
+    die Win32::FormatMessage( Win32::GetLastError() );
+  }
+
+  my $w32pid;
+
+  unless ( $w32pid = $w32job->spawn( $appname, $cmdline ) ) {
+    print $sem_pipe_write "go\n";
+    close $sem_pipe_write;
+    die Win32::FormatMessage( Win32::GetLastError() );
+  }
+
+  print $sem_pipe_write "$w32pid\n";
+  close $sem_pipe_write;
+
+  my $ok = $w32job->watch( sub { 0 }, 60 );
+  my $hashref = $w32job->status();
+  $exitcode = $hashref->{$w32pid}->{exitcode};
+
+  # In case flushing them wasn't good enough.
+  close STDOUT if defined fileno(STDOUT);
+  close STDERR if defined fileno(STDERR);
+
+  exit($exitcode);
 }
 
 1;
@@ -1172,9 +1336,10 @@ POE::Wheel::Run - portably run blocking code and programs in subprocesses
 
 =head1 DESCRIPTION
 
-POE::Wheel::Run executes a program or block of code in a subprocess.
-The parent process may exchange information with the child over the
-child's STDIN, STDOUT and STDERR filehandles.
+POE::Wheel::Run executes a program or block of code in a subprocess,
+created the usual way: using fork().  The parent process may exchange
+information with the child over the child's STDIN, STDOUT and STDERR
+filehandles.
 
 In the parent process, the POE::Wheel::Run object represents the child
 process.  It has methods such as PID() and kill() to query and manage
@@ -1335,6 +1500,10 @@ consistent notification that the child is done sending output.  Please
 note that it does not signal when the child process has exited.
 Programs should use sig_child() to detect that.
 
+While it is impossible for ErrorEvent or StdoutEvent to happen after
+CloseEvent, there is no such guarantee for CHLD, which may happen before
+or after CloseEvent.
+
 In addition to the usual POE parameters, each CloseEvent comes with
 one of its own:
 
@@ -1383,6 +1552,9 @@ A sample error event handler:
     warn "Wheel $wheel_id generated $operation error $errnum: $errstr\n";
   }
 
+Note that unless you deactivate the signal pipe, you might allso see C<EIO>
+(5) error during read operations.
+
 =head4 StdinEvent
 
 StdinEvent contains the name of an event that Wheel::Run emits
@@ -1501,7 +1673,7 @@ See L</NoSetSid>.
 
 =head4 Priority
 
-Priority adjusts the child process' nicenes or priority level,
+Priority adjusts the child process' niceness or priority level,
 depending on which (if any) the underlying OS supports.  Priority
 contains a numeric offset which will be added to the parent's priority
 to determine the child's.
@@ -1524,12 +1696,11 @@ exec(@$program).  As per exec(ARRAY), shell metacharacters will not be
 significant.
 
 If Program holds a code reference, that code will be called in the
-child process.  The child process will exit after that code is
-finished.  This mode allows POE::Wheel::Run to fork off bits of
-long-running code.  Return values, if any, must be passed back via the
-child's STDOUT and/or STDERR.  Note, however, that POE's services are
-effectively disabled in the child process.  See L</Nested POE Kernel>
-for instructions on how to properly use POE within the child.
+child process.  This mode allows POE::Wheel::Run to execute
+long-running internal code asynchronously, while the usual modes
+execute external programs.  The child process will exit after that
+code is finished, in such a way as to avoid DESTROY and END block
+execution.  See L</Coderef Execution Side Effects> for more details.
 
 L<perlfunc> has more information about exec() and the different ways
 to call it.
@@ -1705,25 +1876,54 @@ that performs the exec() call for us.
 That deletes everything from the environment and sets a simple, secure
 PATH before executing a program.
 
-=head2 Nested POE Kernel
+=head2 Coderef Execution Side Effects
 
-The child process is created by fork(), which effectively duplicates
-the parent's POE::Kernel data structures, including its queue and all
-active sessions.
+The child process is created by fork(), which duplicates the parent
+process including a copy of POE::Kernel, all running Session
+instances, events in the queue, watchers, open filehandles, and so on.
 
-If C<< POE::Kernel->run() >> is called again in the child process, it
-effectively resumes a copy of the parent process, which is rarely (if
-ever) the desired effect.
+When executing an external program, the UNIX exec() call immediately
+replaces the copy of the parent with a completely new program.
 
-Likewise, DESTROY methods and END blocks would be triggered if the
-child process simply calls exit().  This is why POE::Wheel::Run takes
-drastic measures to avoid plain old exit() in the child process.
+When executing internal coderefs, however, we must preserve the code
+and any memory it might reference.  This leads to some potential side
+effects.
 
-Some applications require POE to be run in the child process, however.
-If the application wishes to avoid using exec(), it may first stop()
-the POE::Kernel instance in the child, then run() it again.
+=head3 DESTROY and END Blocks Run Twice
 
-Here is an example:
+Objects that were created in the parent process are copied into the
+child.  When the child exits normally, any DESTROY and END blocks are
+executed there.  Later, when the parent exits, they may run again.
+
+POE::Wheel::Run takes steps to avoid running DESTROY and END blocks in
+the child process.  It uses POSIX::_exit() to bypass them.  If that
+fails, it may even kill() itself.
+
+If an application needs to exit explicitly, for example to return an
+error code to the parent process, then please use POSIX::_exit()
+rather than Perl's core exit().
+
+=head3 POE::Kernel's run() method was never called
+
+This warning is displayed from POE::Kernel's DESTROY method.  It's a
+side effect of calling exit() in a child process that was started
+before C<< POE::Kernel->run() >> could be called.  The child process
+receives a copy of POE::Kernel where run() wasn't called, even if it
+was called later in the parent process.
+
+The most direct solution is to call POSIX::_exit() rather than exit().
+This will bypass POE::Kernel's DESTROY, and the message it emits.
+
+=head3 Running POE::Kernel in the Child
+
+Calling c<< POE::Kernel->run() >> in the child process effectively
+resumes the copy of the parent process.  This is rarely (if ever)
+desired.
+
+More commonly, an application wants to run an entirely new POE::Kernel
+instance in the child process.  This is supported by first stop()ping
+the copied instance, starting one or more new sessions, and calling
+run() again.  For example:
 
   Program => sub {
     # Wipe the existing POE::Kernel clean.
@@ -1742,10 +1942,6 @@ Strange things are bound to happen if the program does not call
 L<POE::Kernel/stop> before L<POE::Kernel/run>.  However this is
 vaguely supported in case it's the right thing to do at the time.
 
-The advantage of calling C<POE::Kernel/stop> is that it allows all the
-advantages of a fork() without an exec(), namely sharing of read only
-data, but without having to forefit L<POE>'s facilities in the child.
-
 =head1 SEE ALSO
 
 L<POE::Wheel> describes wheels in general.
